#!/usr/bin/env python3
# This monitoring script was imported from the Hipex platform.
#
# Originally inspired by https://www.monlinux.net/files/check_aacraid.py and rewritten for prometheus

# Grabs the output from "/usr/sbin/arcconf GETCONFIG 1 LD" then
# determines the health of the Logical Devices.
#
# Grabs the output from "/usr/sbin/arcconf GETCONFIG 1 AL" then
# determines the health of various status indicators from the card
# and drives.
#
# Grabs the output from "/usr/sbin/arcconf GETCONFIG 1 PD" then
# determines the health of drives.
#
# After the checks are run, it deletes the file "UcliEvt.log" from
# the current working directory.

import os
import re
import sys
from subprocess import PIPE, Popen

c_status_re = re.compile(r"^\s*Controller Status\s*:\s*(.*)$", re.IGNORECASE)
c_count_re = re.compile(r"^\s* Controller information\s*$", re.IGNORECASE)
l_status_re = re.compile(r"^\s*Status of Logical Device\s*:\s*(.*)$", re.IGNORECASE)
l_device_re = re.compile(r"^Logical Device number ([0-9]+).*$", re.IGNORECASE)
c_defunct_re = re.compile(
    r"^\s*Defunct disk drive count\s:\s*([0-9]+).*$", re.IGNORECASE
)
c_degraded_re = re.compile(
    r"^\s*Logical devices/Failed/Degraded\s*:\s*([0-9]+)/([0-9]+)/([0-9]+).*$",
    re.IGNORECASE,
)
d_device_re = re.compile(r"^\s*Device\s*\s*#([0-9]+)\s*$", re.IGNORECASE)
d_state_re = re.compile(r"^\s*State\s*:\s*(.*)$", re.IGNORECASE)

cstatus = (
    lstatus
) = (
    ldevice
) = cdefunct = cdegraded = bstatus = btemp = bcapacity = btime = ddecide = dstate = ""
lnum = ""
result = ""

for line in Popen(
        ["/usr/sbin/arcconf", "GETCONFIG", "1", "LD"],
        stdin=PIPE,
        stdout=PIPE,
        close_fds=True,
        encoding="utf-8",
).stdout:
    # Match the regexs
    ldevice = l_device_re.match(line)
    if ldevice:
        lnum = ldevice.group(1)
        continue

    lstatus = l_status_re.match(line)
    if lstatus:
        if lstatus.group(1) != "Optimal":
            result += 'arcconf_logical_device_status{device="' + lnum + '"} 0\n'
        else:
            result += 'arcconf_logical_device_status{device="' + lnum + '"} 1\n'

controller = 0
for line in Popen(
        ["/usr/sbin/arcconf", "GETCONFIG", "1", "AD"],
        stdin=PIPE,
        stdout=PIPE,
        close_fds=True,
        encoding="utf-8",
).stdout:
    ccount = c_count_re.match(line)
    if ccount:
        controller = controller + 1

    # Match the regexs
    cstatus = c_status_re.match(line)
    if cstatus:
        if cstatus.group(1) != "Optimal":
            result += (
                    'arcconf_controller_status{controller="' + str(controller) + '"} 0\n'
            )
        else:
            result += (
                    'arcconf_controller_status{controller="' + str(controller) + '"} 1\n'
            )
        continue

    cdefunct = c_defunct_re.match(line)
    if cdefunct:
        result += (
                'arcconf_drives_defunct{controller="'
                + str(controller)
                + '"} '
                + cdefunct.group(1)
                + "\n"
        )
        continue

    cdegraded = c_degraded_re.match(line)
    if cdegraded:
        result += (
                'arcconf_drives_failed{controller="'
                + str(controller)
                + '"} '
                + cdegraded.group(2)
                + "\n"
        )
        result += (
                'arcconf_drives_degraded{controller="'
                + str(controller)
                + '"} '
                + cdegraded.group(3)
                + "\n"
        )
        continue

for line in Popen(
        ["/usr/sbin/arcconf", "GETCONFIG", "1", "PD"],
        stdin=PIPE,
        stdout=PIPE,
        close_fds=True,
        encoding="utf-8",
).stdout:
    # Match the regexs
    ddevice = d_device_re.match(line)
    if ddevice:
        dnum = ddevice.group(1)
        continue

    dstate = d_state_re.match(line)
    if dstate:
        if dstate.group(1) != "Online":
            result += (
                    'arcconf_status{controller="'
                    + str(controller)
                    + '",device="'
                    + dnum
                    + '"} 0\n'
            )
        else:
            result += (
                    'arcconf_status{controller="'
                    + str(controller)
                    + '",device="'
                    + dnum
                    + '"} 1\n'
            )

print(result)

try:
    cwd = os.getcwd()
    fullpath = os.path.join(cwd, "UcliEvt.log")
    os.unlink(fullpath)
except:
    pass

sys.exit(0)
