#!/bin/bash

set -e

getInstances() {
    pgrep -f "php"
}

getPhpPerformanceMetrics() {
    PHP_SCRIPT=$(cat <<'END_HEREDOC'
$benchmarks = [
    'math' => function ($count = 200000) {
        $x = 0;
        for ($i = 0; $i < $count; $i++) {
            $x += $i + $i;
            $x += $i * $i;
            $x += $i ** $i;
            $x += $i / (($i + 1) * 2);
            $x += $i % (($i + 1) * 2);
        }
        return $x;
    },
    'loops' => function ($count = 20000000) {
        for ($i = 0; $i < $count; ++$i) {
            $i;
        }
        $i = 0;
        while ($i < $count) {
            ++$i;
        }
        return $i;
    },
    'ifelse' => function ($count = 10000000) {
        $a = 0;
        $b = 0;
        for ($i = 0; $i < $count; $i++) {
            $k = $i % 4;
            if ($k === 0) {
                $i;
            } elseif ($k === 1) {
                $a = $i;
            } elseif ($k === 2) {
                $b = $i;
            } else {
                $i;
            }
        }
        return $a - $b;
    },
    'switch' => function ($count = 10000000) {
        $a = 0;
        $b = 0;
        for ($i = 0; $i < $count; $i++) {
            switch ($i % 4) {
                case 0:
                    $i;
                    break;
                case 1:
                    $a = $i;
                    break;
                case 2:
                    $b = $i;
                    break;
                default:
                    break;
            }
        }
        return $a - $b;
    },
    'string' => function ($count = 50000) {
        $string = '<i>the</i> quick brown fox jumps over the lazy dog  ';
        for ($i = 0; $i < $count; $i++) {
            addslashes($string);
            bin2hex($string);
            chunk_split($string);
            convert_uudecode(convert_uuencode($string));
            count_chars($string);
            explode(' ', $string);
            htmlentities($string);
            md5($string);
            metaphone($string);
            ord($string);
            rtrim($string);
            sha1($string);
            soundex($string);
            str_getcsv($string);
            str_ireplace('fox', 'cat', $string);
            str_pad($string, 50);
            str_repeat($string, 10);
            str_replace('fox', 'cat', $string);
            str_rot13($string);
            str_shuffle($string);
            str_word_count($string);
            strip_tags($string);
            strpos($string, 'fox');
            strlen($string);
            strtolower($string);
            strtoupper($string);
            substr_count($string, 'the');
            trim($string);
            ucfirst($string);
            ucwords($string);
        }
        return $string;
    },
    'array' => function ($count = 50000) {
        $a = range(0, 100);
        for ($i = 0; $i < $count; $i++) {
            array_keys($a);
            array_values($a);
            array_flip($a);
            array_map(function ($e) {}, $a);
            array_walk($a, function ($e, $i) {});
            array_reverse($a);
            array_sum($a);
            array_merge($a, [101, 102, 103]);
            array_replace($a, [1, 2, 3]);
            array_chunk($a, 2);
        }
        return $a;
    },
    'regex' => function ($count = 1000000) {
        for ($i = 0; $i < $count; $i++) {
            preg_match("#http[s]?://\\w+[^\s\\[\\]\<]+#",
                'this is a link to https://google.com which is a really popular site');
            preg_replace("#(^|\\s)(http[s]?://\\w+[^\s\\[\\]\<]+)#i", '\\1<a href="\\2">\\2</a>',
                'this is a link to https://google.com which is a really popular site');
        }
        return $i;
    },
    'is_type' => function ($count = 2500000) {
        $o = new stdClass();
        for ($i = 0; $i < $count; $i++) {
            is_array([1]);
            is_array('1');
            is_int(1);
            is_int('abc');
            is_string('foo');
            is_string(123);
            is_bool(true);
            is_bool(5);
            is_numeric('hi');
            is_numeric('123');
            is_float(1.3);
            is_float(0);
            is_object($o);
            is_object('hi');
        }
        return $o;
    },
    'hash' => function ($count = 10000) {
        for ($i = 0; $i < $count; $i++) {
            md5($i);
            sha1($i);
            hash('sha256', $i);
            hash('sha512', $i);
            hash('ripemd160', $i);
            hash('crc32', $i);
            hash('crc32b', $i);
            hash('adler32', $i);
            hash('fnv132', $i);
            hash('fnv164', $i);
            hash('joaat', $i);
            hash('haval128,3', $i);
            hash('haval160,3', $i);
            hash('haval192,3', $i);
            hash('haval224,3', $i);
            hash('haval256,3', $i);
            hash('haval128,4', $i);
            hash('haval160,4', $i);
            hash('haval192,4', $i);
            hash('haval224,4', $i);
            hash('haval256,4', $i);
            hash('haval128,5', $i);
            hash('haval160,5', $i);
            hash('haval192,5', $i);
            hash('haval224,5', $i);
            hash('haval256,5', $i);
        }
        return $i;
    },
    'json' => function ($count = 100000) {
        $data = [
            'foo' => 'bar',
            'bar' => 'baz',
            'baz' => 'qux',
            'qux' => 'quux',
            'quux' => 'corge',
            'corge' => 'grault',
            'grault' => 'garply',
            'garply' => 'waldo',
            'waldo' => 'fred',
            'fred' => 'plugh',
            'plugh' => 'xyzzy',
            'xyzzy' => 'thud',
            'thud' => 'end',
        ];
        for ($i = 0; $i < $count; $i++) {
            json_encode($data);
            json_decode(json_encode($data));
        }
        return $data;
    },
];

$results = [];
foreach ($benchmarks as $name => $benchmark) {
    $startTime = microtime(true);
    $benchmark();
    $endTime = microtime(true);
    $results[$name] = $endTime - $startTime;
}

foreach ($results as $name => $time) {
    echo "${name} ${time}\n";
}
END_HEREDOC
    )
    php -r "$PHP_SCRIPT"
}

declare -A totalTimes

TEMP_FILE=$(mktemp)
getPhpPerformanceMetrics > "$TEMP_FILE"

while read -r LINE; do
    STAT=$(echo "${LINE}" | awk '{ print $1 }' | sed 's/{type}/type/g')
    VALUE=$(echo "${LINE}" | awk '{ print $2 }' | sed 's/\r//g')
    if [ -n "${totalTimes[$STAT]}" ]; then
        totalTimes[$STAT]=$(echo "${totalTimes[$STAT]} + ${VALUE}" | bc)
    else
        totalTimes[$STAT]=$VALUE
    fi
done < "$TEMP_FILE"

rm -f "$TEMP_FILE"

totalTime=0
for STAT in "${!totalTimes[@]}"; do
    printf "php_benchmark_%s %.4f\n" "$STAT" "${totalTimes[$STAT]}"
    totalTime=$(echo "${totalTime} + ${totalTimes[$STAT]}" | bc)
done

printf "php_benchmark_total_time %.4f\n" "$totalTime"
